﻿Imports System.Threading
Imports Opto22.Emu

Module Module1

    Sub Main()
        ' Available 32-Bit Integer Values (for Version 1)
        Dim earyInstantaneousValues() As Opto22Emu.eInstantaneousF32Values = New Opto22Emu.eInstantaneousF32Values() _
        { _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_Voltage, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_Current, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_TruePower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_ApparentPower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_PowerFactor, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_Voltage, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_Current, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_TruePower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_ApparentPower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_PowerFactor, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_Voltage, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_Current, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_TruePower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_ApparentPower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_PowerFactor, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_Total_TruePower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_Total_ApparentPower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_Total_AbsoluteTruePower, _
            Opto22Emu.eInstantaneousF32Values.v1_CT_Total_PowerFactor, _
            Opto22Emu.eInstantaneousF32Values.v1_Pulse_A, _
            Opto22Emu.eInstantaneousF32Values.v1_Pulse_B, _
            Opto22Emu.eInstantaneousF32Values.v1_Pulse_C, _
            Opto22Emu.eInstantaneousF32Values.v1_Pulse_D _
        }

        ' Available 64-Bit Double-Precision Values (for Version 1)
        Dim earyDoublePrecisionValues() As Opto22Emu.eAccumulated64Values = _
        { _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseA_Accumulated_TruePower, _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseA_Accumulated_ApparentPower, _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseB_Accumulated_TruePower, _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseB_Accumulated_ApparentPower, _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseC_Accumulated_TruePower, _
            Opto22Emu.eAccumulated64Values.v1_CT_PhaseC_Accumulated_ApparentPower, _
            Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_TruePower, _
            Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_AbsoluteTruePower, _
            Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_ApparentPower, _
            Opto22Emu.eAccumulated64Values.v1_Pulse_A_Total, _
            Opto22Emu.eAccumulated64Values.v1_Pulse_B_Total, _
            Opto22Emu.eAccumulated64Values.v1_Pulse_C_Total, _
            Opto22Emu.eAccumulated64Values.v1_Pulse_D_Total _
        }

        ' create a new object, this initiates a connection with the sensor
        Dim emu As Opto22Emu = New Opto22Emu("10.192.54.49", 2001, False)
        emu.CommunicationTimeoutMs = 1000
        emu.ScanTimeMs = 1000

        ' enables the remote I/O scanning
        emu.RemoteIoScanning = True

        ' enables the scanner to read the output states from the emu
        emu.EnableOutputScanning = True

        ' enables the scanner to write the cached states to the emu
        emu.EnableOutputWriting = True

        ' wait one scan time interval to allow the connection to refresh data
        Thread.Sleep(emu.ScanTimeMs)

        ' emu data version, the currently set scan time, and the communication timeout
        Console.WriteLine("Sensor Data Version {0}", emu.DataVersion)
        Console.WriteLine("Data Scan Time {0}", emu.ScanTimeMs)
        Console.WriteLine("Communication Timeout in Milliseconds {0}", emu.ScanTimeMs)

        Dim i32Counter As Int32 = 0

        ' loop a while
        For j As Int32 = 0 To 29 Step 1
            ' start with the data timestamp and scanning active flags
            Console.WriteLine("Data Timestamp {0}", emu.DataTimeStamp)
            Console.WriteLine("Data Scanning {0}", emu.IsScanning)

            ' print the int32 instantaneous value
            For i As Int32 = 0 To earyInstantaneousValues.Length - 1 Step 1
                Console.WriteLine("{0} {1}", earyInstantaneousValues(i), emu.CurrentValue(earyInstantaneousValues(i)))
            Next

            ' print the accumulated values
            For i As Int32 = 0 To earyDoublePrecisionValues.Length - 1 Step 1
                Console.WriteLine("{0} {1}", earyDoublePrecisionValues(i), emu.AccumulatedValueAsF64(earyDoublePrecisionValues(i)))
            Next

            ' read remote I/O values
            For i As Int32 = 0 To Opto22Emu.MaximumRemoteIoValues - 1 Step 1
                Dim f32Measurement As Single = 0.0
                Dim f32Status As Single = 0.0
                Dim f32Freshness As Single = 0.0
                Dim i64Measurement As Int64 = 0
                Dim i64Status As Int64 = 0
                Dim i64Freshness As Int64 = 0
                Dim bSuccess As Boolean

                ' read the values as floats values
                bSuccess = emu.RetrieveRemoteIoValues(i, f32Measurement, f32Status, f32Freshness)
                Console.WriteLine("Float I/O Values Index {0} Measurement {1} Status {2} Freshness {3} Method Status {4}.", i, f32Measurement, f32Status, f32Freshness, bSuccess)

                ' read the values as signed integers
                bSuccess = emu.RetrieveRemoteIoValues(i, i64Measurement, i64Status, i64Freshness)
                Console.WriteLine("Integer I/O Values Index {0} Measurement {1} Status {2} Freshness {3} Method Status {4}.", i, i64Measurement, i64Status, i64Freshness, bSuccess)
            Next

            Dim state As Opto22Emu.eOutputState

            ' read DR output states (only valid if a DR and EnableOutputScanning evaluates true)
            For i As Int32 = 0 To Opto22Emu.MaximumOutputValues - 1 Step 1
                state = emu.GetOutputValue(i)
                Console.WriteLine("DR Output State {0} reports {1}.", i, state)
            Next

            ' toggle a DR output state
            state = emu.GetOutputValue(i32Counter Mod Opto22Emu.MaximumOutputValues)
            If state = Opto22Emu.eOutputState.Off Then
                state = Opto22Emu.eOutputState.On
            Else
                state = Opto22Emu.eOutputState.Off
            End If

            ' set the output state outputs will only update if EnableOutputWriting evaluates true
            emu.SetOutputValue(i32Counter Mod Opto22Emu.MaximumOutputValues, state)

            ' increment the counter
            i32Counter = i32Counter + 1

            ' sleep a while because data scans don't occur that often so wait at least the scan time before reading values again
            Thread.Sleep(emu.ScanTimeMs)

            ' blank line
            Console.WriteLine()

            ' when the Opto22Emu object is demolished, the background thread will be automatically terminated and cleaned up
        Next

        ' you have to do this to stop the background thread or the window will disappear but the app will remain
        ' running in the task manager application list!!!!
        emu.Stop()

    End Sub

End Module
