﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Opto22.Emu;

namespace OptoEmu_Sensor_MMP
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e)
		{

		}

		/// <summary>
		/// Opto MMP Port of the EMU (usually it's 2001 but may change if a NAT router or someone changes this location on the EMU)
		/// </summary>
		Int32 i32OptoMmpPort = 0;

		/// <summary>
		/// Communication timeout in milliseconds. For local networks, 1000 ms; for more latent networks, larger times.
		/// </summary>
		Int32 i32CommunicationTimeoutMs = 0;

		/// <summary>
		/// Scantime. The EMU updates about every 2 seconds. So a value of 2000 ms is considered a minimum.
		/// </summary>
		Int32 i32ScanTimeMs = 0;

		/// <summary>
		/// The emu object.
		/// </summary>
		Opto22Emu emu = null;

		private void buttonConnect_Click(object sender, EventArgs e)
		{
			if (emu != null)
			{
				timerUpdateData.Stop();
				emu.Stop();
				textConnectionStatus.Text = emu.IsScannerConnected == true ? "Connected" : "Not Connected";
				textConnectionStatus.BackColor = emu.IsScannerConnected ? Color.LimeGreen : Color.Red;
				textScanningStatus.Text = emu.IsScanning == true ? "Running" : "Not Running";
				textScanningStatus.BackColor = emu.IsScanning ? Color.LimeGreen : Color.Red;
				emu = null;
				buttonConnect.Text = "Connect To Emu";
			}
			else
			{
				// retrieve the form information, port, timeout, and scan time
				i32OptoMmpPort = Int32.Parse(textOptoMmpPort.Text);
				i32CommunicationTimeoutMs = Int32.Parse(textTimeoutMs.Text);
				i32ScanTimeMs = Int32.Parse(textScanTimeMs.Text);
				buttonConnect.Text = "Disconnect From Emu";

				// open the connection as the form shows
				emu = new Opto22Emu(textIpAddressOrHost.Text, i32OptoMmpPort, checkUseIpv6.Checked, checkBoxUseTcp.Checked);
				timerUpdateData_Tick(sender, e);
				timerUpdateData.Interval = i32ScanTimeMs / 2;
				timerUpdateData.Start();
			}
		}

		private void timerUpdateData_Tick(object sender, EventArgs e)
		{
			// update the form with version 1 values
			textPhaseAVoltage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_Voltage).ToString("F1");
			textPhaseBVoltage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_Voltage).ToString("F1");
			textPhaseCVoltage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_Voltage).ToString("F1");

			textPhaseACurrent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_Current).ToString("F2");
			textPhaseBCurrent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_Current).ToString("F2");
			textPhaseCCurrent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_Current).ToString("F2");

			textPhaseATruePower.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_TruePower).ToString("F1");
			textPhaseBTruePower.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_TruePower).ToString("F1");
			textPhaseCTruePower.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_TruePower).ToString("F1");

			textPhaseAApparent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_ApparentPower).ToString("F1");
			textPhaseBApparent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_ApparentPower).ToString("F1");
			textPhaseCApparent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_ApparentPower).ToString("F1");

			textPhaseAPowerFactor.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseA_PowerFactor).ToString("F2");
			textPhaseBPowerFactor.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseB_PowerFactor).ToString("F2");
			textPhaseCPowerFactor.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_PhaseC_PowerFactor).ToString("F2");

			textPhaseAAccumulatedTruePower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseA_Accumulated_TruePower).ToString("F1");
			textPhaseBAccumulatedTruePower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseB_Accumulated_TruePower).ToString("F1");
			textPhaseCAccumulatedTruePower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseC_Accumulated_TruePower).ToString("F1");

			textPhaseAAccumulatedApparent.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseA_Accumulated_ApparentPower).ToString("F1");
			textPhaseBAccumulatedApparent.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseB_Accumulated_ApparentPower).ToString("F1");
			textPhaseCAccumulatedApparent.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_PhaseC_Accumulated_ApparentPower).ToString("F1");

			textTotalTruePower.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_Total_TruePower).ToString("F1");
			textTotalApparent.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_Total_ApparentPower).ToString("F1");
			textTotalAbsolutePower.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_Total_AbsoluteTruePower).ToString("F1");
			textTotalPowerFactor.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_CT_Total_PowerFactor).ToString("F2");

			textTotalAccumulatedTruePower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_TruePower).ToString("F1");
			textTotalAccumulatedAbsolutePower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_AbsoluteTruePower).ToString("F1");
			textTotalAccumulatedApparentPower.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_CT_Total_Accumulated_ApparentPower).ToString("F1");

			textPulseAUsage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_Pulse_A).ToString("F1");
			textPulseBUsage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_Pulse_B).ToString("F1");
			textPulseCUsage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_Pulse_C).ToString("F1");
			textPulseDUsage.Text = emu.CurrentValue(Opto22Emu.eInstantaneousF32Values.v1_Pulse_D).ToString("F1");

			textPulseATotal.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_Pulse_A_Total).ToString("F1");
			textPulseBTotal.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_Pulse_B_Total).ToString("F1");
			textPulseCTotal.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_Pulse_C_Total).ToString("F1");
			textPulseDTotal.Text = emu.AccumulatedValueAsF64(Opto22Emu.eAccumulated64Values.v1_Pulse_D_Total).ToString("F1");

			textDataTimeStamp.Text = emu.DataTimeStamp.ToString();

			textConnectionStatus.Text = emu.IsScannerConnected == true ? "Connected" : "Not Connected";
			textConnectionStatus.BackColor = emu.IsScannerConnected ? Color.LimeGreen : Color.Red;
			textScanningStatus.Text = emu.IsScanning == true ? "Running" : "Not Running";
			textScanningStatus.BackColor = emu.IsScanning ? Color.LimeGreen : Color.Red;

			// test if remote IO scanning is enabled; this is an EMU version 2 feature
			if (emu.RemoteIoScanning == true)
			{
				// configure the datagridview
				dataGridViewRemoteInputs.Enabled = true;
				dataGridViewRemoteInputs.ColumnCount = 6;
				dataGridViewRemoteInputs.Columns[0].Name = "Index";
				dataGridViewRemoteInputs.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[0].Width = 40;
				dataGridViewRemoteInputs.Columns[1].Name = "Float";
				dataGridViewRemoteInputs.Columns[1].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[1].Width = 60;
				dataGridViewRemoteInputs.Columns[2].Name = "Signed Integer";
				dataGridViewRemoteInputs.Columns[2].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[2].Width = 60;
				dataGridViewRemoteInputs.Columns[3].Name = "Unsigned Integer";
				dataGridViewRemoteInputs.Columns[3].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[3].Width = 60;
				dataGridViewRemoteInputs.Columns[4].Name = "I64 Status";
				dataGridViewRemoteInputs.Columns[4].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[4].Width = 60;
				dataGridViewRemoteInputs.Columns[5].Name = "I64 Freshness";
				dataGridViewRemoteInputs.Columns[5].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
				dataGridViewRemoteInputs.Columns[5].Width = 60;

				// set the number of rows
				dataGridViewRemoteInputs.RowCount = emu.MaximumRemoteIoValues;

				// populate values
				for (Int32 i = 0; i < emu.MaximumRemoteIoValues; i++)
				{
					Single f32Measurement = (Single)0.0;
					Single f32Status = (Single)0.0;
					Single f32Freshness = (Single)0.0;
					Int64 i64Measurement = 0;
					Int32 i32Measurement = 0;
					UInt32 u32Measurement = 0;
					Int64 i64Status = 0; // ignored
					Int64 i64Freshness = 0; // ignored
					Boolean bSuccess; // ignored

					// retrieve the remote IO Values, the only way these methods return false is if bSuccess == false (note, I'm not checking it here).
					bSuccess = emu.RetrieveRemoteIoValues(i, out f32Measurement, out f32Status, out f32Freshness);
					bSuccess = emu.RetrieveRemoteIoValues(i, out i64Measurement, out i64Status, out i64Freshness);

					// cast values; the i64 values are really either a signed or unsigned 32-bit integer
					i32Measurement = (Int32)i64Measurement;
					u32Measurement = (UInt32)i64Measurement;

					// update the datagridview
					dataGridViewRemoteInputs.Rows[i].Cells[0].Value = i.ToString();
					dataGridViewRemoteInputs.Rows[i].Cells[1].Value = f32Measurement.ToString("F2");
					dataGridViewRemoteInputs.Rows[i].Cells[2].Value = i32Measurement.ToString();
					dataGridViewRemoteInputs.Rows[i].Cells[3].Value = u32Measurement.ToString();
					dataGridViewRemoteInputs.Rows[i].Cells[4].Value = i64Status.ToString();
					dataGridViewRemoteInputs.Rows[i].Cells[5].Value = i64Freshness.ToString();
				}
			}
			else
			{
				// ensure the datagridview is disabled
				dataGridViewRemoteInputs.Enabled = false;
			}
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			// when the form closes initiate a shutdown on the background scanning thread
			if(emu != null)
				emu.Stop();
		}

		private void checkBoxRemoteIoEnabled_CheckedChanged(object sender, EventArgs e)
		{
			// update the RemoteIoScanning mode
			emu.RemoteIoScanning = checkBoxRemoteIoEnabled.Checked;

			// if the remote IO scanning was disabled, clear the datagridview to hide values
			if (emu.RemoteIoScanning == false)
			{
				// these settings clear and disable the datagridview
				dataGridViewRemoteInputs.Enabled = false;
				dataGridViewRemoteInputs.RowCount = 0;
				dataGridViewRemoteInputs.ColumnCount = 0;
			}
		}
	}
}
